@@ -6,7 +6,8 @@ import typing
 import getpass
 import logging
 import threading
-from typing import Dict, List, Callable, Any, Optional, Type
+from typing import Any, Dict, List, Type, Callable, Optional, DefaultDict
+from collections import defaultdict
 
 from telegram import VERSION
 from telegram.utils import AsyncResult
@@ -16,22 +17,30 @@ from telegram.worker import BaseWorker, SimpleWorker
 logger = logging.getLogger(__name__)
 
 
-class Telegram(object):
-    def __init__(self,
-                 api_id: int,
-                 api_hash: str,
-                 phone: str,
-                 database_encryption_key: str,
-                 library_path: str = None,
-                 worker: Optional[Type[BaseWorker]] = None,
-                 files_directory: str = None,
-                 use_test_dc: bool = False,
-                 use_message_database: bool = True,
-                 device_model: str = 'python-telegram',
-                 application_version: str = VERSION,
-                 system_version: str = 'unknown',
-                 system_language_code: str = 'en',
-                 login: bool = False) -> None:
+MESSAGE_HANDLER_TYPE: str = 'updateNewMessage'
+
+
+class Telegram:
+    def __init__(
+        self,
+        api_id: int,
+        api_hash: str,
+        database_encryption_key: str,
+        phone: str = None,
+        bot_token: str = None,
+        library_path: str = None,
+        worker: Optional[Type[BaseWorker]] = None,
+        files_directory: str = None,
+        use_test_dc: bool = False,
+        use_message_database: bool = True,
+        device_model: str = 'python-telegram',
+        application_version: str = VERSION,
+        system_version: str = 'unknown',
+        system_language_code: str = 'en',
+        login: bool = False,
+        default_workers_queue_size=1000,
+        tdlib_verbosity: int = 2,
+    ) -> None:
         """
         Args:
             api_id - ID of your app (https://my.telegram.org/apps/)
@@ -51,12 +60,17 @@ class Telegram(object):
         self.api_hash = api_hash
         self.library_path = library_path
         self.phone = phone
+        self.bot_token = bot_token
         self.use_test_dc = use_test_dc
         self.device_model = device_model
         self.system_version = system_version
         self.system_language_code = system_language_code
         self.application_version = application_version
         self.use_message_database = use_message_database
+        self._queue_put_timeout = 10
+
+        if not self.bot_token and not self.phone:
+            raise ValueError('You must provide bot_token or phone')
 
         self._database_encryption_key = database_encryption_key
 
@@ -66,26 +80,32 @@ class Telegram(object):
 
         self._authorized = False
         self._is_enabled = False
-        self._queue: queue.Queue = queue.Queue()
 
-        self._workers_queue: queue.Queue = queue.Queue()
+        # todo: move to worker
+        self._workers_queue: queue.Queue = queue.Queue(
+            maxsize=default_workers_queue_size
+        )
+
         if not worker:
             worker = SimpleWorker
         self.worker = worker(queue=self._workers_queue)
 
         self._results: Dict[str, AsyncResult] = {}
-        self._message_handlers: List[Callable] = []
-        self._update_handlers: List[Callable] = []
+        self._update_handlers: DefaultDict[str, List[Callable]] = defaultdict(list)
 
-        self._tdjson = TDJson(
-            library_path=library_path,
-        )
+        self._tdjson = TDJson(library_path=library_path, verbosity=tdlib_verbosity)
         self._run()
 
         if login:
             self.login()
 
     def __del__(self):
+        self.stop()
+
+    def stop(self) -> None:
+        """Stops the client"""
+        self._is_enabled = False
+
         if hasattr(self, '_tdjson'):
             self._tdjson.stop()
 
@@ -115,12 +135,10 @@ class Telegram(object):
             'chat_id': chat_id,
             'input_message_content': {
                 '@type': 'inputMessageText',
-                'text': {
-                    '@type': 'formattedText',
-                    'text': text,
-                },
+                'text': {'@type': 'formattedText', 'text': text},
             },
         }
+
         return self._send_data(data)
 
     def get_chat(self, chat_id: int) -> AsyncResult:
@@ -128,16 +146,22 @@ class Telegram(object):
         This is offline request, if there is no chat in your database it will not be found
         tdlib saves chat to the database when it receives a new message or when you call `get_chats` method.
         """
-        data = {
-            '@type': 'getChat',
-            'chat_id': chat_id,
-        }
+        data = {'@type': 'getChat', 'chat_id': chat_id}
+
         return self._send_data(data)
 
-    def get_chats(self,
-                  offset_order: int = 0,
-                  offset_chat_id: int = 0,
-                  limit: int = 100) -> AsyncResult:
+    def get_me(self) -> AsyncResult:
+        """
+        Requests information of the current user (getMe method)
+
+        https://core.telegram.org/tdlib/docs/classtd_1_1td__api_1_1get_me.html
+        """
+
+        return self.call_method('getMe')
+
+    def get_chats(
+        self, offset_order: int = 0, offset_chat_id: int = 0, limit: int = 100
+    ) -> AsyncResult:
         """
         Returns a list of chats:
 
@@ -156,14 +180,17 @@ class Telegram(object):
             'offset_chat_id': offset_chat_id,
             'limit': limit,
         }
+
         return self._send_data(data)
 
-    def get_chat_history(self,
-                         chat_id: int,
-                         limit: int = 1000,
-                         from_message_id: int = 0,
-                         offset: int = 0,
-                         only_local: bool = False):
+    def get_chat_history(
+        self,
+        chat_id: int,
+        limit: int = 1000,
+        from_message_id: int = 0,
+        offset: int = 0,
+        only_local: bool = False,
+    ):
         """
         Returns history of a chat
 
@@ -182,6 +209,7 @@ class Telegram(object):
             'offset': offset,
             'only_local': only_local,
         }
+
         return self._send_data(data)
 
     def get_web_page_instant_view(self, url: str, force_full: bool = False):
@@ -193,14 +221,11 @@ class Telegram(object):
             url: URL of a webpage
             force_full: If true, the full instant view for the web page will be returned
         """
-        data = {
-            '@type': 'getWebPageInstantView',
-            'url': url,
-            'force_full': force_full,
-        }
+        data = {'@type': 'getWebPageInstantView', 'url': url, 'force_full': force_full}
+
         return self._send_data(data)
 
-    def call_method(self, method_name: str, params: Dict[str, Any]):
+    def call_method(self, method_name: str, params: Optional[Dict[str, Any]] = None):
         """
         Use this method to call any other method of the tdlib
 
@@ -208,10 +233,11 @@ class Telegram(object):
             method_name: Name of the method
             params: parameters
         """
-        data = {
-            '@type': method_name,
-        }
-        data.update(params)
+        data = {'@type': method_name}
+
+        if params:
+            data.update(params)
+
         return self._send_data(data)
 
     def _run(self):
@@ -225,17 +251,21 @@ class Telegram(object):
 
     def _listen_to_td(self):
         logger.info('[Telegram.td_listener] started')
+
         while self._is_enabled:
             update = self._tdjson.receive()
+
             if update:
-                self._queue.put(update)
                 self._update_async_result(update)
                 self._run_handlers(update)
 
     def _update_async_result(self, update: dict) -> typing.Optional[AsyncResult]:
         async_result = None
 
-        _special_types = ('updateAuthorizationState', )  # for authorizationProcess @extra.request_id doesn't work
+        _special_types = (
+            'updateAuthorizationState',
+        )  # for authorizationProcess @extra.request_id doesn't work
+
         if update.get('@type') in _special_types:
             request_id = update['@type']
         else:
@@ -247,20 +277,26 @@ class Telegram(object):
             async_result = self._results.get(request_id)
 
         if not async_result:
-            logger.debug(f'async_result has not been found in by request_id={request_id}')
+            logger.debug(
+                'async_result has not been found in by request_id=%s', request_id
+            )
         else:
-            async_result._parse_update(update)
+            async_result.parse_update(update)
+            self._results.pop(request_id, None)
 
         return async_result
 
     def _run_handlers(self, update: Dict[Any, Any]) -> None:
-        if update.get('@type') == 'updateNewMessage':
-            for handler in self._message_handlers:
-                self._workers_queue.put((handler, update))
+        update_type: str = update.get('@type', 'unknown')
+        for handler in self._update_handlers[update_type]:
+            self._workers_queue.put((handler, update), timeout=self._queue_put_timeout)
 
     def add_message_handler(self, func: Callable) -> None:
-        if func not in self._message_handlers:
-            self._message_handlers.append(func)
+        self.add_update_handler(MESSAGE_HANDLER_TYPE, func)
+
+    def add_update_handler(self, handler_type: str, func: Callable) -> None:
+        if func not in self._update_handlers[handler_type]:
+            self._update_handlers[handler_type].append(func)
 
     def _send_data(self, data: dict, result_id: str = None) -> AsyncResult:
         if '@extra' not in data:
@@ -269,26 +305,25 @@ class Telegram(object):
         if not result_id and 'request_id' in data['@extra']:
             result_id = data['@extra']['request_id']
 
-        async_result = AsyncResult(
-            client=self,
-            result_id=result_id,
-        )
+        async_result = AsyncResult(client=self, result_id=result_id)
         data['@extra']['request_id'] = async_result.id
 
         self._tdjson.send(data)
         self._results[async_result.id] = async_result
         async_result.request = data
+
         return async_result
 
     def idle(self, stop_signals=(signal.SIGINT, signal.SIGTERM, signal.SIGABRT)):
         """Blocks until one of the signals are received and stops"""
+
         for sig in stop_signals:
             signal.signal(sig, self._signal_handler)
 
         self._is_enabled = True
 
         while self._is_enabled:
-            time.sleep(1)
+            time.sleep(0.1)
 
     def _signal_handler(self, signum, frame):
         self._is_enabled = False
@@ -304,22 +339,30 @@ class Telegram(object):
             None: self._set_initial_params,
             'authorizationStateWaitTdlibParameters': self._set_initial_params,
             'authorizationStateWaitEncryptionKey': self._send_encryption_key,
-            'authorizationStateWaitPhoneNumber': self._send_phone_number,
+            'authorizationStateWaitPhoneNumber': self._send_phone_number_or_bot_token,
             'authorizationStateWaitCode': self._send_telegram_code,
             'authorizationStateWaitPassword': self._send_password,
             'authorizationStateReady': self._complete_authorization,
         }
-        logger.info('[login] Login process has been started')
+        if self.phone:
+            logger.info('[login] Login process has been started with phone')
+        else:
+            logger.info('[login] Login process has been started with bot token')
 
         while not self._authorized:
-            logger.info(f'[login] current authorization state: {authorization_state}')
+            logger.info('[login] current authorization state: %s', authorization_state)
             result = actions[authorization_state]()
+
             if result:
                 result.wait(raise_exc=True)
                 authorization_state = result.update['authorization_state']['@type']
 
     def _set_initial_params(self) -> AsyncResult:
-        logger.info(f'Setting tdlib initial params: files_dir={self.files_directory} test_dc={self.use_test_dc}')
+        logger.info(
+            'Setting tdlib initial params: files_dir=%s, test_dc=%s',
+            self.files_directory,
+            self.use_test_dc,
+        )
         data = {
             # todo: params
             '@type': 'setTdlibParameters',
@@ -334,8 +377,9 @@ class Telegram(object):
                 'database_directory': os.path.join(self.files_directory, 'database'),
                 'use_message_database': self.use_message_database,
                 'files_directory': os.path.join(self.files_directory, 'files'),
-            }
+            },
         }
+
         return self._send_data(data, result_id='updateAuthorizationState')
 
     def _send_encryption_key(self) -> AsyncResult:
@@ -344,8 +388,18 @@ class Telegram(object):
             '@type': 'checkDatabaseEncryptionKey',
             'encryption_key': self._database_encryption_key,
         }
+
         return self._send_data(data, result_id='updateAuthorizationState')
 
+    def _send_phone_number_or_bot_token(self) -> AsyncResult:
+        """Sends phone number or a bot_token"""
+        if self.phone:
+            return self._send_phone_number()
+        elif self.bot_token:
+            return self._send_bot_token()
+        else:
+            raise RuntimeError('Unknown mode: both bot_token and phone are None')
+
     def _send_phone_number(self) -> AsyncResult:
         logger.info('Sending phone number')
         data = {
@@ -354,24 +408,27 @@ class Telegram(object):
             'allow_flash_call': False,
             'is_current_phone_number': True,
         }
+
+        return self._send_data(data, result_id='updateAuthorizationState')
+
+    def _send_bot_token(self) -> AsyncResult:
+        logger.info('Sending bot token')
+        data = {'@type': 'checkAuthenticationBotToken', 'token': self.bot_token}
+
         return self._send_data(data, result_id='updateAuthorizationState')
 
     def _send_telegram_code(self) -> AsyncResult:
         logger.info('Sending code')
         code = input('Enter code:')
-        data = {
-            '@type': 'checkAuthenticationCode',
-            'code': str(code),
-        }
+        data = {'@type': 'checkAuthenticationCode', 'code': str(code)}
+
         return self._send_data(data, result_id='updateAuthorizationState')
 
     def _send_password(self) -> AsyncResult:
         logger.info('Sending password')
         password = getpass.getpass('Password:')
-        data = {
-            '@type': 'checkAuthenticationPassword',
-            'password': password,
-        }
+        data = {'@type': 'checkAuthenticationPassword', 'password': password}
+
         return self._send_data(data, result_id='updateAuthorizationState')
 
     def _complete_authorization(self) -> None:
