@@ -1,42 +1,38 @@
 import json
 import logging
 import platform
-from ctypes import (
-    CDLL,
-    CFUNCTYPE,
-    c_int,
-    c_char_p,
-    c_double,
-    c_void_p,
-    c_longlong,
-)
-from typing import Any, Dict
+from ctypes import CDLL, CFUNCTYPE, c_int, c_char_p, c_double, c_void_p, c_longlong
+from typing import Any, Dict, Optional
 
 import pkg_resources
 
 logger = logging.getLogger(__name__)
 
 
-def _get_tdjson_lib_path():
+def _get_tdjson_lib_path() -> str:
     if platform.system().lower() == 'darwin':
         lib_name = 'darwin/libtdjson.dylib'
     else:
         lib_name = 'linux/libtdjson.so'
-    return pkg_resources.resource_filename(
-        'telegram',
-        f'lib/{lib_name}',
-    )
+
+    return pkg_resources.resource_filename('telegram', f'lib/{lib_name}')
 
 
-class TDJson(object):
-    def __init__(self, library_path: str = None) -> None:
+class TDJson:
+    def __init__(self, library_path: Optional[str] = None, verbosity: int = 2) -> None:
         if library_path is None:
             library_path = _get_tdjson_lib_path()
-        logger.info(f'Using shared library "{library_path}"')
+        logger.info('Using shared library "%s"', library_path)
 
-        self._build_client(library_path)
+        self._build_client(library_path, verbosity)
 
-    def _build_client(self, library_path: str):
+    def __del__(self):
+        if hasattr(self, '_tdjson') and hasattr(
+            self._tdjson, '_td_json_client_destroy'
+        ):
+            self.stop()
+
+    def _build_client(self, library_path: str, verbosity: int) -> None:
         self._tdjson = CDLL(library_path)
 
         # load TDLib functions from shared library
@@ -74,17 +70,19 @@ class TDJson(object):
         self._td_set_log_verbosity_level.restype = None
         self._td_set_log_verbosity_level.argtypes = [c_int]
 
-        self._td_set_log_verbosity_level(2)
+        self._td_set_log_verbosity_level(verbosity)
 
         fatal_error_callback_type = CFUNCTYPE(None, c_char_p)
 
-        self._td_set_log_fatal_error_callback = self._tdjson.td_set_log_fatal_error_callback
+        self._td_set_log_fatal_error_callback = (
+            self._tdjson.td_set_log_fatal_error_callback
+        )
         self._td_set_log_fatal_error_callback.restype = None
         self._td_set_log_fatal_error_callback.argtypes = [fatal_error_callback_type]
 
         # initialize TDLib log with desired parameters
-        def on_fatal_error_callback(error_message):
-            logger.error('TDLib fatal error: ', error_message)
+        def on_fatal_error_callback(error_message: str) -> None:
+            logger.error('TDLib fatal error: %s', error_message)
 
         c_on_fatal_error_callback = fatal_error_callback_type(on_fatal_error_callback)
         self._td_set_log_fatal_error_callback(c_on_fatal_error_callback)
@@ -92,21 +90,27 @@ class TDJson(object):
     def send(self, query: Dict[Any, Any]) -> None:
         dumped_query = json.dumps(query).encode('utf-8')
         self._td_json_client_send(self.td_json_client, dumped_query)
-        logger.debug(f'[me ==>] Sent {dumped_query}')
+        logger.debug('[me ==>] Sent %s', dumped_query)
 
     def receive(self) -> Dict[Any, Any]:
         result = self._td_json_client_receive(self.td_json_client, 1.0)
+
         if result:
             result = json.loads(result.decode('utf-8'))
-            logger.debug(f'[me <==] Received {result}')
+            logger.debug('[me <==] Received %s', result)
+
         return result
 
     def td_execute(self, query: Dict[Any, Any]) -> Dict[Any, Any]:
         dumped_query = json.dumps(query).encode('utf-8')
         result = self._td_json_client_execute(self.td_json_client, dumped_query)
+
         if result:
             result = json.loads(result.decode('utf-8'))
+
         return result
 
     def stop(self) -> None:
-        self._tdjson._td_json_client_destroy(self.td_json_client)
+        self._tdjson._td_json_client_destroy(
+            self.td_json_client
+        )  # pylint: disable=protected-access
