@@ -1,13 +1,18 @@
-import uuid
 import time
+import uuid
+from typing import TYPE_CHECKING, Any, Dict, Optional
+
+if TYPE_CHECKING:
+    from telegram.client import Telegram    # noqa  pylint: disable=cyclic-import
 
 
-class AsyncResult(object):
+class AsyncResult:
     """
     tdlib is asynchronous, and this class helps you get results back.
     After each API call, you receive AsyncResult object, which you can use to get results back.
     """
-    def __init__(self, client, result_id=None):
+
+    def __init__(self, client: 'Telegram', result_id=None) -> None:
         self.client = client
 
         if result_id:
@@ -15,11 +20,11 @@ class AsyncResult(object):
         else:
             self.id = uuid.uuid4().hex
 
-        self.request = None
+        self.request: Optional[Dict[Any, Any]] = None
         self.ok_received = False
         self.error = False
-        self.error_info = None
-        self.update = None
+        self.error_info: Optional[Dict[Any, Any]] = None
+        self.update: Optional[Dict[Any, Any]] = None
 
     def __str__(self):
         return f'AsyncResult <{self.id}>'
@@ -38,7 +43,7 @@ class AsyncResult(object):
             if timeout and time.time() - started_at > timeout:
                 raise TimeoutError()
 
-    def _parse_update(self, update: dict) -> None:
+    def parse_update(self, update: dict) -> None:
         if update.get('@type') == 'ok':
             self.ok_received = True
         elif update.get('@type') == 'error':
